@@ -11,10 +11,15 @@
 # express or implied. See the License for the specific language governing
 # permissions and limitations under the License.
 
-# Relative imports
 from ._base import Evaluator, MultivariateEvaluator
+from .backtest import make_evaluation_predictions, backtest_metrics
 
-__all__ = ["Evaluator", "MultivariateEvaluator"]
+__all__ = [
+    "Evaluator",
+    "MultivariateEvaluator",
+    "make_evaluation_predictions",
+    "backtest_metrics",
+]
 
 # fix Sphinx issues, see https://bit.ly/2K2eptM
 for item in __all__:
