@@ -11,7 +11,6 @@
 # express or implied. See the License for the specific language governing
 # permissions and limitations under the License.
 
-# Standard library imports
 import logging
 import multiprocessing
 import sys
@@ -29,14 +28,24 @@ from typing import (
     Union,
 )
 
-# Third-party imports
 import numpy as np
 import pandas as pd
 
+from .metrics import (
+    abs_error,
+    abs_target_mean,
+    abs_target_sum,
+    calculate_seasonal_error,
+    coverage,
+    mape,
+    mase,
+    mse,
+    msis,
+    owa,
+    quantile_loss,
+    smape,
+)
 from gluonts.gluonts_tqdm import tqdm
-from gluonts.time_feature import get_seasonality
-
-# First-party imports
 from gluonts.model.forecast import Forecast, Quantile
 
 
@@ -45,6 +54,51 @@ def worker_function(evaluator: "Evaluator", inp: tuple):
     return evaluator.get_metrics_per_ts(ts, forecast)
 
 
+def aggregate_all(
+    metric_per_ts: pd.DataFrame, agg_funs: Dict[str, str]
+) -> Dict[str, float]:
+    """
+    No filtering applied.
+
+    Both `nan` and `inf` possible in aggregate metrics.
+    """
+    return {
+        key: metric_per_ts[key].agg(agg, skipna=False)
+        for key, agg in agg_funs.items()
+    }
+
+
+def aggregate_no_nan(
+    metric_per_ts: pd.DataFrame, agg_funs: Dict[str, str]
+) -> Dict[str, float]:
+    """
+    Filter all `nan` but keep `inf`.
+
+    `nan` is only possible in the aggregate metric if all timeseries
+    for a metric resulted in `nan`.
+    """
+    return {
+        key: metric_per_ts[key].agg(agg, skipna=True)
+        for key, agg in agg_funs.items()
+    }
+
+
+def aggregate_valid(
+    metric_per_ts: pd.DataFrame, agg_funs: Dict[str, str]
+) -> Dict[str, Union[float, np.ma.core.MaskedConstant]]:
+    """
+    Filter all `nan` & `inf` values from `metric_per_ts`.
+
+    If all metrics in a column of `metric_per_ts` are `nan` or `inf` the
+    result will be `np.ma.masked` for that column.
+    """
+    metric_per_ts = metric_per_ts.apply(np.ma.masked_invalid)
+    return {
+        key: metric_per_ts[key].agg(agg, skipna=True)
+        for key, agg in agg_funs.items()
+    }
+
+
 class Evaluator:
     """
     Evaluator class, to compute accuracy metrics by comparing observations
@@ -83,16 +137,21 @@ class Evaluator:
         E.g. {"RMSE": [rmse, "mean", "median"]}
     num_workers
         The number of multiprocessing workers that will be used to process
-        the data in parallel.
-        Default is multiprocessing.cpu_count().
-        Setting it to 0 means no multiprocessing.
+        the data in parallel. Default is multiprocessing.cpu_count().
+        Setting it to 0 or None means no multiprocessing.
     chunk_size
         Controls the approximate chunk size each workers handles at a time.
         Default is 32.
+    ignore_invalid_values
+        Ignore `NaN` and `inf` values in the timeseries when calculating metrics.
+    aggregation_strategy:
+        Function for aggregating per timeseries metrics.
+        Available options are:
+        aggregate_valid | aggregate_all | aggregate_no_nan
+        The default function is aggregate_no_nan.
     """
 
     default_quantiles = 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9
-    zero_tol = 1e-8
 
     def __init__(
         self,
@@ -101,21 +160,20 @@ class Evaluator:
         alpha: float = 0.05,
         calculate_owa: bool = False,
         custom_eval_fn: Optional[Dict] = None,
-        num_workers: Optional[int] = None,
-        chunk_size: Optional[int] = None,
+        num_workers: Optional[int] = multiprocessing.cpu_count(),
+        chunk_size: int = 32,
+        aggregation_strategy: Callable = aggregate_no_nan,
+        ignore_invalid_values: bool = True,
     ) -> None:
         self.quantiles = tuple(map(Quantile.parse, quantiles))
         self.seasonality = seasonality
         self.alpha = alpha
         self.calculate_owa = calculate_owa
         self.custom_eval_fn = custom_eval_fn
-
-        self.num_workers = (
-            num_workers
-            if num_workers is not None
-            else multiprocessing.cpu_count()
-        )
-        self.chunk_size = chunk_size if chunk_size is not None else 32
+        self.num_workers = num_workers
+        self.chunk_size = chunk_size
+        self.aggregation_strategy = aggregation_strategy
+        self.ignore_invalid_values = ignore_invalid_values
 
     def __call__(
         self,
@@ -153,7 +211,7 @@ class Evaluator:
             total=num_series,
             desc="Running evaluation",
         ) as it, np.errstate(invalid="ignore"):
-            if self.num_workers > 0 and not sys.platform == "win32":
+            if self.num_workers and not sys.platform == "win32":
                 mp_pool = multiprocessing.Pool(
                     initializer=None, processes=self.num_workers
                 )
@@ -249,65 +307,38 @@ class Evaluator:
             np.squeeze(time_series.loc[:date_before_forecast].transpose())
         )
 
-    def seasonal_error(
-        self, past_data: np.ndarray, forecast: Forecast
-    ) -> float:
-        r"""
-        .. math::
-
-            seasonal_error = mean(|Y[t] - Y[t-m]|)
-
-        where m is the seasonal frequency
-        https://www.m4.unic.ac.cy/wp-content/uploads/2018/03/M4-Competitors-Guide.pdf
-        """
-        # Check if the length of the time series is larger than the seasonal frequency
-        seasonality = (
-            self.seasonality
-            if self.seasonality
-            else get_seasonality(forecast.freq)
-        )
-        if seasonality < len(past_data):
-            forecast_freq = seasonality
-        else:
-            # edge case: the seasonal freq is larger than the length of ts
-            # revert to freq=1
-            # logging.info('The seasonal frequency is larger than the length of the time series. Reverting to freq=1.')
-            forecast_freq = 1
-        y_t = past_data[:-forecast_freq]
-        y_tm = past_data[forecast_freq:]
-
-        seasonal_mae = np.mean(abs(y_t - y_tm))
-
-        return seasonal_mae if seasonal_mae is not np.ma.masked else np.nan
-
     def get_metrics_per_ts(
         self, time_series: Union[pd.Series, pd.DataFrame], forecast: Forecast
-    ) -> Dict[str, Union[float, str, None]]:
+    ) -> Dict[str, Union[float, str, None, np.ma.core.MaskedConstant]]:
         pred_target = np.array(self.extract_pred_target(time_series, forecast))
-        pred_target = np.ma.masked_invalid(pred_target)
-
-        # required for seasonal_error and owa calculation
         past_data = np.array(self.extract_past_data(time_series, forecast))
-        past_data = np.ma.masked_invalid(past_data)
+
+        if self.ignore_invalid_values:
+            past_data = np.ma.masked_invalid(past_data)
+            pred_target = np.ma.masked_invalid(pred_target)
 
         try:
-            mean_fcst = forecast.mean
-        except:
+            mean_fcst = getattr(forecast, "mean", None)
+        except NotImplementedError:
             mean_fcst = None
+
         median_fcst = forecast.quantile(0.5)
-        seasonal_error = self.seasonal_error(past_data, forecast)
-        metrics = {
+        seasonal_error = calculate_seasonal_error(
+            past_data, forecast, self.seasonality
+        )
+
+        metrics: Dict[str, Union[float, str, None]] = {
             "item_id": forecast.item_id,
-            "MSE": self.mse(pred_target, mean_fcst)
+            "MSE": mse(pred_target, mean_fcst)
             if mean_fcst is not None
             else None,
-            "abs_error": self.abs_error(pred_target, median_fcst),
-            "abs_target_sum": self.abs_target_sum(pred_target),
-            "abs_target_mean": self.abs_target_mean(pred_target),
+            "abs_error": abs_error(pred_target, median_fcst),
+            "abs_target_sum": abs_target_sum(pred_target),
+            "abs_target_mean": abs_target_mean(pred_target),
             "seasonal_error": seasonal_error,
-            "MASE": self.mase(pred_target, median_fcst, seasonal_error),
-            "MAPE": self.mape(pred_target, median_fcst),
-            "sMAPE": self.smape(pred_target, median_fcst),
+            "MASE": mase(pred_target, median_fcst, seasonal_error),
+            "MAPE": mape(pred_target, median_fcst),
+            "sMAPE": smape(pred_target, median_fcst),
             "OWA": np.nan,  # by default not calculated
         }
 
@@ -318,7 +349,7 @@ class Evaluator:
                         target_fcst = mean_fcst
                     else:
                         logging.warning(
-                            "mean_fcst is None, therfore median_fcst is used."
+                            "mean_fcst is None, therefore median_fcst is used."
                         )
                         target_fcst = median_fcst
                 else:
@@ -337,7 +368,7 @@ class Evaluator:
                 metrics.update(val)
 
         try:
-            metrics["MSIS"] = self.msis(
+            metrics["MSIS"] = msis(
                 pred_target,
                 forecast.quantile(self.alpha / 2),
                 forecast.quantile(1.0 - self.alpha / 2),
@@ -349,7 +380,7 @@ class Evaluator:
             metrics["MSIS"] = np.nan
 
         if self.calculate_owa:
-            metrics["OWA"] = self.owa(
+            metrics["OWA"] = owa(
                 pred_target,
                 median_fcst,
                 past_data,
@@ -360,10 +391,10 @@ class Evaluator:
         for quantile in self.quantiles:
             forecast_quantile = forecast.quantile(quantile.value)
 
-            metrics[quantile.loss_name] = self.quantile_loss(
+            metrics[quantile.loss_name] = quantile_loss(
                 pred_target, forecast_quantile, quantile.value
             )
-            metrics[quantile.coverage_name] = self.coverage(
+            metrics[quantile.coverage_name] = coverage(
                 pred_target, forecast_quantile
             )
 
@@ -372,6 +403,7 @@ class Evaluator:
     def get_aggregate_metrics(
         self, metric_per_ts: pd.DataFrame
     ) -> Tuple[Dict[str, float], pd.DataFrame]:
+
         agg_funs = {
             "MSE": "mean",
             "abs_error": "sum",
@@ -385,54 +417,41 @@ class Evaluator:
             "MSIS": "mean",
         }
 
-        if self.custom_eval_fn is not None:
-            for k, (_, agg_type, _) in self.custom_eval_fn.items():
-                agg_funs.update({k: agg_type})
-
         for quantile in self.quantiles:
             agg_funs[quantile.loss_name] = "sum"
             agg_funs[quantile.coverage_name] = "mean"
 
+        if self.custom_eval_fn is not None:
+            for k, (_, agg_type, _) in self.custom_eval_fn.items():
+                agg_funs.update({k: agg_type})
+
         assert (
             set(metric_per_ts.columns) >= agg_funs.keys()
         ), "Some of the requested item metrics are missing."
 
-        totals = {
-            key: metric_per_ts[key].agg(agg) for key, agg in agg_funs.items()
-        }
+        totals = self.aggregation_strategy(
+            metric_per_ts=metric_per_ts, agg_funs=agg_funs
+        )
 
         # derived metrics based on previous aggregate metrics
         totals["RMSE"] = np.sqrt(totals["MSE"])
-
-        flag = totals["abs_target_mean"] <= Evaluator.zero_tol
-        totals["NRMSE"] = np.divide(
-            totals["RMSE"] * (1 - flag), totals["abs_target_mean"] + flag
-        )
-
-        flag = totals["abs_target_sum"] <= Evaluator.zero_tol
-        totals["ND"] = np.divide(
-            totals["abs_error"] * (1 - flag), totals["abs_target_sum"] + flag
-        )
-
-        all_qLoss_names = [
-            quantile.weighted_loss_name for quantile in self.quantiles
-        ]
-
-        all_abs_qLoss_names = [
-            quantile.loss_name for quantile in self.quantiles
-        ]
+        totals["NRMSE"] = totals["RMSE"] / totals["abs_target_mean"]
+        totals["ND"] = totals["abs_error"] / totals["abs_target_sum"]
 
         for quantile in self.quantiles:
-            totals[quantile.weighted_loss_name] = np.divide(
-                totals[quantile.loss_name], totals["abs_target_sum"] + flag
+            totals[quantile.weighted_loss_name] = (
+                totals[quantile.loss_name] / totals["abs_target_sum"]
             )
 
         totals["mean_absolute_QuantileLoss"] = np.array(
-            [totals[abs_ql] for abs_ql in all_abs_qLoss_names]
+            [totals[quantile.loss_name] for quantile in self.quantiles]
         ).mean()
 
         totals["mean_wQuantileLoss"] = np.array(
-            [totals[ql] for ql in all_qLoss_names]
+            [
+                totals[quantile.weighted_loss_name]
+                for quantile in self.quantiles
+            ]
         ).mean()
 
         totals["MAE_Coverage"] = np.mean(
@@ -443,144 +462,6 @@ class Evaluator:
         )
         return totals, metric_per_ts
 
-    @staticmethod
-    def mse(target, forecast):
-        return np.mean(np.square(target - forecast))
-
-    @staticmethod
-    def abs_error(target, forecast):
-        return np.sum(np.abs(target - forecast))
-
-    @staticmethod
-    def quantile_loss(target, quantile_forecast, q):
-        return 2.0 * np.sum(
-            np.abs(
-                (quantile_forecast - target)
-                * ((target <= quantile_forecast) - q)
-            )
-        )
-
-    @staticmethod
-    def coverage(target, quantile_forecast):
-        return np.mean((target < quantile_forecast))
-
-    @staticmethod
-    def mase(target, forecast, seasonal_error):
-        r"""
-        .. math::
-
-            mase = mean(|Y - Y_hat|) / seasonal_error
-
-        https://www.m4.unic.ac.cy/wp-content/uploads/2018/03/M4-Competitors-Guide.pdf
-        """
-        flag = seasonal_error <= Evaluator.zero_tol
-        return (np.mean(np.abs(target - forecast)) * (1 - flag)) / (
-            seasonal_error + flag
-        )
-
-    @staticmethod
-    def mape(target, forecast):
-        r"""
-        .. math::
-
-            mape = mean(|Y - Y_hat| / |Y|))
-        """
-
-        denominator = np.abs(target)
-        flag = denominator <= Evaluator.zero_tol
-
-        mape = np.mean(
-            (np.abs(target - forecast) * (1 - flag)) / (denominator + flag)
-        )
-        return mape
-
-    @staticmethod
-    def smape(target, forecast):
-        r"""
-        .. math::
-
-            smape = mean(2 * |Y - Y_hat| / (|Y| + |Y_hat|))
-
-        https://www.m4.unic.ac.cy/wp-content/uploads/2018/03/M4-Competitors-Guide.pdf
-        """
-
-        denominator = np.abs(target) + np.abs(forecast)
-        flag = denominator <= Evaluator.zero_tol
-
-        smape = 2 * np.mean(
-            (np.abs(target - forecast) * (1 - flag)) / (denominator + flag)
-        )
-        return smape
-
-    @staticmethod
-    def owa(
-        target: np.ndarray,
-        forecast: np.ndarray,
-        past_data: np.ndarray,
-        seasonal_error: float,
-        start_date: pd.Timestamp,
-    ) -> float:
-        r"""
-        .. math::
-
-            owa = 0.5*(smape/smape_naive + mase/mase_naive)
-
-        https://www.m4.unic.ac.cy/wp-content/uploads/2018/03/M4-Competitors-Guide.pdf
-        """
-        # avoid import error due to circular dependency
-        from gluonts.model.naive_2 import naive_2
-
-        # calculate the forecast of the seasonal naive predictor
-        naive_median_fcst = naive_2(
-            past_data, len(target), freq=start_date.freqstr
-        )
-
-        owa = 0.5 * (
-            (
-                Evaluator.smape(target, forecast)
-                / Evaluator.smape(target, naive_median_fcst)
-            )
-            + (
-                Evaluator.mase(target, forecast, seasonal_error)
-                / Evaluator.mase(target, naive_median_fcst, seasonal_error)
-            )
-        )
-
-        return owa
-
-    @staticmethod
-    def msis(target, lower_quantile, upper_quantile, seasonal_error, alpha):
-        r"""
-        :math:
-
-            msis = mean(U - L + 2/alpha * (L-Y) * I[Y<L] + 2/alpha * (Y-U) * I[Y>U]) /seasonal_error
-
-        https://www.m4.unic.ac.cy/wp-content/uploads/2018/03/M4-Competitors-Guide.pdf
-        """
-        numerator = np.mean(
-            upper_quantile
-            - lower_quantile
-            + 2.0
-            / alpha
-            * (lower_quantile - target)
-            * (target < lower_quantile)
-            + 2.0
-            / alpha
-            * (target - upper_quantile)
-            * (target > upper_quantile)
-        )
-
-        flag = seasonal_error <= Evaluator.zero_tol
-        return (numerator * (1 - flag)) / (seasonal_error + flag)
-
-    @staticmethod
-    def abs_target_sum(target):
-        return np.sum(np.abs(target))
-
-    @staticmethod
-    def abs_target_mean(target):
-        return np.mean(np.abs(target))
-
 
 class MultivariateEvaluator(Evaluator):
     """
@@ -619,6 +500,7 @@ class MultivariateEvaluator(Evaluator):
         eval_dims: List[int] = None,
         target_agg_funcs: Dict[str, Callable] = {},
         custom_eval_fn: Optional[dict] = None,
+        num_workers: Optional[int] = None,
     ) -> None:
         """
 
@@ -640,12 +522,16 @@ class MultivariateEvaluator(Evaluator):
             pass key-value pairs that define aggregation functions over the
             dimension axis. Useful to compute metrics over aggregated target
             and forecast (typically sum or mean).
+        num_workers
+            The number of multiprocessing workers that will be used to process
+            metric for each dimension of the multivariate forecast.
         """
         super().__init__(
             quantiles=quantiles,
             seasonality=seasonality,
             alpha=alpha,
             custom_eval_fn=custom_eval_fn,
+            num_workers=num_workers,
         )
         self._eval_dims = eval_dims
         self.target_agg_funcs = target_agg_funcs
