@@ -15,10 +15,9 @@ from typing import Iterator, List, Optional, Tuple
 
 import numpy as np
 
-from gluonts.core.component import DType, validated
+from gluonts.core.component import DType, validated, tensor_to_numpy
 from gluonts.core.exception import assert_data_error
 from gluonts.dataset.common import DataEntry
-from gluonts.model.common import Tensor
 from gluonts.support.util import erf, erfinv
 
 from ._base import (
@@ -293,7 +292,7 @@ class SampleTargetDim(FlatMapTransformation):
         self.shuffle = shuffle
 
     def flatmap_transform(
-        self, data: DataEntry, is_train: bool, slice_future_target: bool = True
+        self, data: DataEntry, is_train: bool
     ) -> Iterator[DataEntry]:
         if not is_train:
             yield data
@@ -338,6 +337,7 @@ class CDFtoGaussianTransform(MapTransformation):
         observed_values_field: str,
         cdf_suffix="_cdf",
         max_context_length: Optional[int] = None,
+        dtype: DType = np.float32,
     ) -> None:
         """
         Constructor for CDFtoGaussianTransform.
@@ -354,6 +354,8 @@ class CDFtoGaussianTransform(MapTransformation):
             Suffix to mark the field with the transformed target.
         max_context_length
             Sets the maximum context length for the empirical CDF.
+        dtype
+            numpy dtype of output.
         """
         self.target_field = target_field
         self.past_target_field = "past_" + self.target_field
@@ -365,6 +367,7 @@ class CDFtoGaussianTransform(MapTransformation):
         self.cdf_suffix = cdf_suffix
         self.max_context_length = max_context_length
         self.target_dim = target_dim
+        self.dtype = dtype
 
     def map_transform(self, data: DataEntry, is_train: bool) -> DataEntry:
         self._preprocess_data(data, is_train=is_train)
@@ -435,7 +438,9 @@ class CDFtoGaussianTransform(MapTransformation):
         # sorts along the time dimension to compute empirical CDF of each
         # dimension
         if is_train:
-            past_target_vec = self._add_noise(past_target_vec)
+            past_target_vec = self._add_noise(past_target_vec).astype(
+                self.dtype
+            )
 
         past_target_vec.sort(axis=0)
 
@@ -464,10 +469,13 @@ class CDFtoGaussianTransform(MapTransformation):
         sorted_target = data[self.sort_target_field]
         sorted_target_length, target_dim = sorted_target.shape
 
-        quantiles = np.stack(
-            [np.arange(sorted_target_length) for _ in range(target_dim)],
-            axis=1,
-        ) / float(sorted_target_length)
+        quantiles = (
+            np.stack(
+                [np.arange(sorted_target_length) for _ in range(target_dim)],
+                axis=1,
+            )
+            / float(sorted_target_length)
+        )
 
         x_diff = np.diff(sorted_target, axis=0)
         y_diff = np.diff(quantiles, axis=0)
@@ -484,8 +492,8 @@ class CDFtoGaussianTransform(MapTransformation):
         intercepts = quantiles - slopes * sorted_target
 
         # Populate new fields with the piece-wise linear parameters.
-        data[self.slopes_field] = slopes
-        data[self.intercepts_field] = intercepts
+        data[self.slopes_field] = slopes.astype(self.dtype)
+        data[self.intercepts_field] = intercepts.astype(self.dtype)
 
     def _empirical_cdf_forward_transform(
         self,
@@ -612,12 +620,12 @@ class CDFtoGaussianTransform(MapTransformation):
     @staticmethod
     def standard_gaussian_cdf(x: np.array) -> np.array:
         u = x / (np.sqrt(2.0))
-        return (erf(np, u) + 1.0) / 2.0
+        return (erf(u) + 1.0) / 2.0
 
     @staticmethod
     def standard_gaussian_ppf(y: np.array) -> np.array:
         y_clipped = np.clip(y, a_min=1.0e-6, a_max=1.0 - 1.0e-6)
-        return np.sqrt(2.0) * erfinv(np, 2.0 * y_clipped - 1.0)
+        return np.sqrt(2.0) * erfinv(2.0 * y_clipped - 1.0)
 
     @staticmethod
     def winsorized_cutoff(m: np.array) -> np.array:
@@ -676,7 +684,7 @@ class CDFtoGaussianTransform(MapTransformation):
 
 
 def cdf_to_gaussian_forward_transform(
-    input_batch: DataEntry, outputs: Tensor
+    input_batch: DataEntry, outputs: np.ndarray
 ) -> np.ndarray:
     """
     Forward transformation of the CDFtoGaussianTransform.
@@ -695,10 +703,10 @@ def cdf_to_gaussian_forward_transform(
     """
 
     def _empirical_cdf_inverse_transform(
-        batch_target_sorted: Tensor,
-        batch_predictions: Tensor,
-        slopes: Tensor,
-        intercepts: Tensor,
+        batch_target_sorted: np.ndarray,
+        batch_predictions: np.ndarray,
+        slopes: np.ndarray,
+        intercepts: np.ndarray,
     ) -> np.ndarray:
         """
         Apply forward transformation of the empirical CDF.
@@ -720,10 +728,7 @@ def cdf_to_gaussian_forward_transform(
             Forward transformed outputs.
 
         """
-        slopes = slopes.asnumpy()
-        intercepts = intercepts.asnumpy()
 
-        batch_target_sorted = batch_target_sorted.asnumpy()
         batch_size, num_timesteps, target_dim = batch_target_sorted.shape
         indices = np.floor(batch_predictions * num_timesteps)
         # indices = indices - 1
@@ -746,11 +751,11 @@ def cdf_to_gaussian_forward_transform(
     batch_size, samples, target_dim, time = outputs.shape
     for sample_index in range(0, samples):
         outputs[:, sample_index, :, :] = _empirical_cdf_inverse_transform(
-            input_batch["past_target_sorted"],
+            tensor_to_numpy(input_batch["past_target_sorted"]),
             CDFtoGaussianTransform.standard_gaussian_cdf(
                 outputs[:, sample_index, :, :]
             ),
-            input_batch["slopes"],
-            input_batch["intercepts"],
+            tensor_to_numpy(input_batch["slopes"]),
+            tensor_to_numpy(input_batch["intercepts"]),
         )
     return outputs
