@@ -11,31 +11,29 @@
 # express or implied. See the License for the specific language governing
 # permissions and limitations under the License.
 
-# Standard library imports
-from typing import Tuple
+from typing import Tuple, List
+
+import mxnet as mx
 
-# Third-party imports
 import numpy as np
 import pandas as pd
-import mxnet as mx
 import pytest
 
-# First-party imports
 import gluonts
+import gluonts.mx.component
 from gluonts import time_feature, transform
 from gluonts.core import fqname_for
 from gluonts.core.serde import dump_code, dump_json, load_code, load_json
-from gluonts.dataset.common import ProcessStartField, DataEntry, ListDataset
+from gluonts.dataset.common import DataEntry, ListDataset, ProcessStartField
 from gluonts.dataset.field_names import FieldName
 from gluonts.dataset.stat import ScaleHistogram, calculate_dataset_statistics
-
 from gluonts.transform import (
-    MissingValueImputation,
-    LeavesMissingValues,
-    DummyValueImputation,
-    MeanValueImputation,
-    LastValueImputation,
     CausalMeanValueImputation,
+    DummyValueImputation,
+    LastValueImputation,
+    LeavesMissingValues,
+    MeanValueImputation,
+    MissingValueImputation,
     RollingMeanValueImputation,
 )
 
@@ -226,12 +224,21 @@ def test_InstanceSplitter(
         is_pad_field=FieldName.IS_PAD,
         start_field=FieldName.START,
         forecast_start_field=FieldName.FORECAST_START,
-        train_sampler=transform.UniformSplitSampler(p=1.0),
+        instance_sampler=(
+            transform.UniformSplitSampler(
+                p=1.0,
+                min_past=0 if pick_incomplete else train_length,
+                min_future=lead_time + pred_length,
+            )
+            if is_train
+            else transform.TestSplitSampler(
+                min_past=0 if pick_incomplete else train_length
+            )
+        ),
         past_length=train_length,
         future_length=pred_length,
         lead_time=lead_time,
         time_series_fields=["some_time_feature"],
-        pick_incomplete=pick_incomplete,
     )
 
     assert_serializable(t)
@@ -306,7 +313,18 @@ def test_CanonicalInstanceSplitter(
         is_pad_field=FieldName.IS_PAD,
         start_field=FieldName.START,
         forecast_start_field=FieldName.FORECAST_START,
-        instance_sampler=transform.UniformSplitSampler(p=1.0),
+        instance_sampler=(
+            transform.UniformSplitSampler(
+                p=1.0,
+                min_past=train_length,
+            )
+            if is_train
+            else (
+                transform.ValidationSplitSampler()
+                if allow_target_padding
+                else transform.TestSplitSampler()
+            )
+        ),
         instance_length=train_length,
         prediction_length=pred_length,
         time_series_fields=["some_time_feature"],
@@ -326,7 +344,7 @@ def test_CanonicalInstanceSplitter(
 
     out = list(t.flatmap_transform(data, is_train=is_train))
 
-    min_num_instances = 1 if allow_target_padding else 0
+    min_num_instances = 1 if allow_target_padding and not is_train else 0
     if is_train:
         assert len(out) == max(
             min_num_instances, len(target) - train_length + 1
@@ -387,7 +405,7 @@ def test_Transformation():
                 is_pad_field=FieldName.IS_PAD,
                 start_field=FieldName.START,
                 forecast_start_field=FieldName.FORECAST_START,
-                train_sampler=transform.ExpectedNumInstanceSampler(
+                instance_sampler=transform.ExpectedNumInstanceSampler(
                     num_instances=4
                 ),
                 past_length=train_length,
@@ -458,8 +476,12 @@ def test_multi_dim_transformation(is_train):
                 is_pad_field=FieldName.IS_PAD,
                 start_field=FieldName.START,
                 forecast_start_field=FieldName.FORECAST_START,
-                train_sampler=transform.ExpectedNumInstanceSampler(
-                    num_instances=4
+                instance_sampler=(
+                    transform.ExpectedNumInstanceSampler(
+                        num_instances=4, min_future=pred_length
+                    )
+                    if is_train
+                    else transform.TestSplitSampler()
                 ),
                 past_length=train_length,
                 future_length=pred_length,
@@ -520,12 +542,11 @@ def test_ExpectedNumInstanceSampler():
                 is_pad_field=FieldName.IS_PAD,
                 start_field=FieldName.START,
                 forecast_start_field=FieldName.FORECAST_START,
-                train_sampler=transform.ExpectedNumInstanceSampler(
-                    num_instances=4
+                instance_sampler=transform.ExpectedNumInstanceSampler(
+                    num_instances=4, min_future=pred_length
                 ),
                 past_length=train_length,
                 future_length=pred_length,
-                pick_incomplete=True,
             )
         ]
     )
@@ -562,12 +583,11 @@ def test_BucketInstanceSampler():
                 is_pad_field=FieldName.IS_PAD,
                 start_field=FieldName.START,
                 forecast_start_field=FieldName.FORECAST_START,
-                train_sampler=transform.BucketInstanceSampler(
-                    dataset_stats.scale_histogram
+                instance_sampler=transform.BucketInstanceSampler(
+                    scale_histogram=dataset_stats.scale_histogram
                 ),
                 past_length=train_length,
                 future_length=pred_length,
-                pick_incomplete=True,
             )
         ]
     )
@@ -766,12 +786,10 @@ def point_process_dataset():
 
 
 class MockContinuousTimeSampler(transform.ContinuousTimePointSampler):
-    # noinspection PyMissingConstructor,PyUnusedLocal
-    def __init__(self, ret_values, *args, **kwargs):
-        self._ret_values = ret_values
+    ret_values: List[float]
 
     def __call__(self, *args, **kwargs):
-        return np.array(self._ret_values)
+        return np.array(self.ret_values)
 
 
 @pytest.fixture
@@ -783,9 +801,13 @@ def test_ctsplitter_mask_sorted(point_process_dataset):
     ts = np.cumsum(ia_times)
 
     splitter = transform.ContinuousTimeInstanceSplitter(
-        2,
-        1,
-        train_sampler=transform.ContinuousTimeUniformSampler(num_instances=10),
+        past_interval_length=2,
+        future_interval_length=1,
+        instance_sampler=transform.ContinuousTimeUniformSampler(
+            num_instances=10,
+            min_past=2,
+            min_future=1,
+        ),
     )
 
     # no boundary conditions
@@ -799,9 +821,12 @@ def test_ctsplitter_mask_sorted(point_process_dataset):
 
 def test_ctsplitter_no_train_last_point(point_process_dataset):
     splitter = transform.ContinuousTimeInstanceSplitter(
-        2,
-        1,
-        train_sampler=transform.ContinuousTimeUniformSampler(num_instances=10),
+        past_interval_length=2,
+        future_interval_length=1,
+        instance_sampler=transform.ContinuousTimePredictionSampler(
+            allow_empty_interval=False,
+            min_past=2,
+        ),
     )
 
     iter_de = splitter(point_process_dataset, is_train=False)
@@ -821,10 +846,10 @@ def test_ctsplitter_no_train_last_point(point_process_dataset):
 
 def test_ctsplitter_train_correct(point_process_dataset):
     splitter = transform.ContinuousTimeInstanceSplitter(
-        1,
-        1,
-        train_sampler=MockContinuousTimeSampler(
-            ret_values=[1.01, 1.5, 1.99], num_instances=3
+        past_interval_length=1,
+        future_interval_length=1,
+        instance_sampler=MockContinuousTimeSampler(
+            ret_values=[1.01, 1.5, 1.99]
         ),
     )
 
@@ -860,10 +885,10 @@ def test_ctsplitter_train_correct_out_count(point_process_dataset):
                 yield d
 
     splitter = transform.ContinuousTimeInstanceSplitter(
-        1,
-        1,
-        train_sampler=MockContinuousTimeSampler(
-            ret_values=[1.01, 1.5, 1.99], num_instances=3
+        past_interval_length=1,
+        future_interval_length=1,
+        instance_sampler=MockContinuousTimeSampler(
+            ret_values=[1.01, 1.5, 1.99]
         ),
     )
 
@@ -877,7 +902,13 @@ def test_ctsplitter_train_correct_out_count(point_process_dataset):
 def test_ctsplitter_train_samples_correct_times(point_process_dataset):
 
     splitter = transform.ContinuousTimeInstanceSplitter(
-        1.25, 1.25, train_sampler=transform.ContinuousTimeUniformSampler(20)
+        past_interval_length=1.25,
+        future_interval_length=1.25,
+        instance_sampler=transform.ContinuousTimeUniformSampler(
+            num_instances=20,
+            min_past=1.25,
+            min_future=1.25,
+        ),
     )
 
     iter_de = splitter(point_process_dataset, is_train=True)
@@ -898,8 +929,8 @@ def test_ctsplitter_train_short_intervals(point_process_dataset):
     splitter = transform.ContinuousTimeInstanceSplitter(
         0.01,
         0.01,
-        train_sampler=MockContinuousTimeSampler(
-            ret_values=[1.01, 1.5, 1.99], num_instances=3
+        instance_sampler=MockContinuousTimeSampler(
+            ret_values=[1.01, 1.5, 1.99]
         ),
     )
 
